/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "data.h"

/**
 * @brief Set a type of maps view
 * @param[in] ad The application data
 * @param[in] type The application data
 */
static void set_map_view_type(appdata_s *ad, maps_view_type_e type)
{
	maps_view_set_type(ad->map_view_handle, type);

	maps_coordinates_h coordinates;
	maps_view_get_center(ad->map_view_handle, &coordinates);
	maps_view_set_center(ad->map_view_handle, coordinates);
	maps_coordinates_destroy(coordinates);
}

/**
 * @brief Move the ctx popup
 * @param[in] ctxpopup The object of ctx popup
 */
static void move_more_ctxpopup(Evas_Object *ctxpopup)
{
	Evas_Object *win;
	Evas_Coord w, h;
	int pos = -1;

	/* We convince the top widget is a window */
	win = elm_object_top_widget_get(ctxpopup);
	elm_win_screen_size_get(win, NULL, NULL, &w, &h);
	pos = elm_win_rotation_get(win);

	switch (pos) {
	case 0:
	case 180:
		evas_object_move(ctxpopup, (w / 2), h);
		break;
	case 90:
		evas_object_move(ctxpopup,  (h / 2), w);
		break;
	case 270:
		evas_object_move(ctxpopup, (h / 2), w);
		break;
	}
}

/**
 * @brief Function will be operated when the ctxpopup is dismissed.
 * @param[in] data The data to be passed to the callback function
 * @param[in] obj The Evas object handle to be passed to the callback function
 * @param[in] event_info The system event information
 */
static void ctxpopup_dismissed_cb(void *data, Evas_Object *obj, void *event_info)
{
	evas_object_del(obj);
	obj = NULL;
}

/**
 * @brief Function will be operated when the naviframe is resized.
 * @param[in] data The data to be passed to the callback function
 * @param[in] obj The Evas object handle to be passed to the callback function
 * @param[in] event_info The system event information
 */
static void naviframe_resize_cb(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	Evas_Object *ctxpopup = data;
	move_more_ctxpopup(ctxpopup);
}

/**
 * @brief Function will be operated when the win is rotated.
 * @param[in] data The data to be passed to the callback function
 * @param[in] obj The Evas object handle to be passed to the callback function
 * @param[in] event_info The system event information
 */
static void win_rotation_changed_cb(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *ctxpopup = data;
	move_more_ctxpopup(ctxpopup);
}

/**
 * @brief Function will be operated when the ctxpopup is deleted.
 * @param[in] data The data to be passed to the callback function
 * @param[in] obj The Evas object handle to be passed to the callback function
 * @param[in] event_info The system event information
 */
static void more_ctxpopup_del_cb(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	Evas_Object *nf = data;
	evas_object_event_callback_del_full(nf, EVAS_CALLBACK_RESIZE, naviframe_resize_cb, obj);
}

/**
 * @brief Function will be operated when the item of ctxpopup about theme is selected.
 * @param[in] data The data to be passed to the callback function
 * @param[in] obj The Evas object handle to be passed to the callback function
 * @param[in] event_info The system event information
 */
static void change_map_theme_cb(void *data, Evas_Object *obj, void *event_info)
{
	appdata_s *ad = evas_object_data_get(obj, "app_data");

	set_map_view_type(ad, (maps_view_type_e)data);
	evas_object_del(ad->ctxpopup);
}

/**
 * @brief Function will be operated when the item of ctxpopup about traffic is selected.
 * @param[in] data The data to be passed to the callback function
 * @param[in] obj The Evas object handle to be passed to the callback function
 * @param[in] event_info The system event information
 */
static void change_map_traffic_cb(void *data, Evas_Object *obj, void *event_info)
{
	appdata_s *ad = evas_object_data_get(obj, "app_data");
	maps_coordinates_h coordinates = NULL;

	maps_view_set_traffic_enabled(ad->map_view_handle, (bool)data);
	maps_view_get_center(ad->map_view_handle, &coordinates);
	maps_view_set_center(ad->map_view_handle, coordinates);
	maps_coordinates_destroy(coordinates);

	evas_object_del(obj);
}

/**
 * @brief Function will be operated when the item of ctxpopup about transit is selected.
 * @param[in] data The data to be passed to the callback function
 * @param[in] obj The Evas object handle to be passed to the callback function
 * @param[in] event_info The system event information
 */
static void change_map_public_transit_cb(void *data, Evas_Object *obj, void *event_info)
{
	appdata_s *ad = evas_object_data_get(obj, "app_data");
	maps_coordinates_h coordinates = NULL;

	maps_view_set_public_transit_enabled(ad->map_view_handle, (bool)data);
	maps_view_get_center(ad->map_view_handle, &coordinates);
	maps_view_set_center(ad->map_view_handle, coordinates);
	maps_coordinates_destroy(coordinates);

	evas_object_del(obj);
}

/**
 * @brief Function will be operated when the more button is selected.
 * @param[in] data The data to be passed to the callback function
 * @param[in] obj The Evas object handle to be passed to the callback function
 * @param[in] event_info The system event information
 */
void create_ctxpopup_cb(void *data, Evas_Object *obj, void *event_info)
{
	appdata_s *ad = (appdata_s *)data;

	if (ad->ctxpopup != NULL)
		evas_object_del(ad->ctxpopup);

	/*
	 * CTX POPUP
	 * Create and initialize elm_ctxpopup.
	 */
	ad->ctxpopup = elm_ctxpopup_add(ad->nf);
	elm_ctxpopup_auto_hide_disabled_set(ad->ctxpopup, EINA_TRUE);
	elm_object_style_set(ad->ctxpopup, "more/default");
	evas_object_data_set(ad->ctxpopup, "app_data", ad);
	eext_object_event_callback_add(ad->ctxpopup, EEXT_CALLBACK_BACK, eext_ctxpopup_back_cb, NULL);
	eext_object_event_callback_add(ad->ctxpopup, EEXT_CALLBACK_MORE, eext_ctxpopup_back_cb, NULL);
	evas_object_smart_callback_add(ad->ctxpopup, "dismissed", ctxpopup_dismissed_cb, NULL);
	evas_object_event_callback_add(ad->ctxpopup, EVAS_CALLBACK_DEL, more_ctxpopup_del_cb, ad->nf);

	evas_object_event_callback_add(ad->nf, EVAS_CALLBACK_RESIZE, naviframe_resize_cb, ad->ctxpopup);
	evas_object_smart_callback_add(ad->win, "rotation,changed", win_rotation_changed_cb, ad->ctxpopup);

	/* append item to ctxpopup about theme */
	elm_ctxpopup_item_append(ad->ctxpopup, "Normal map", NULL, change_map_theme_cb, (void *)MAPS_VIEW_TYPE_NORMAL);
	elm_ctxpopup_item_append(ad->ctxpopup, "Satellite map", NULL, change_map_theme_cb, (void *)MAPS_VIEW_TYPE_SATELLITE);
	elm_ctxpopup_item_append(ad->ctxpopup, "Terrain map", NULL, change_map_theme_cb, (void *)MAPS_VIEW_TYPE_TERRAIN);
	elm_ctxpopup_item_append(ad->ctxpopup, "Hybrid map", NULL, change_map_theme_cb, (void *)MAPS_VIEW_TYPE_HYBRID);

	bool traffic_enabled = false;
	int ret = 0;
	ret = maps_view_get_traffic_enabled(ad->map_view_handle, &traffic_enabled);
	if (ret == MAPS_ERROR_NONE) {
		/* append item to ctxpopup about traffic */
		if (traffic_enabled)
			elm_ctxpopup_item_append(ad->ctxpopup, "Turn off Traffic", NULL, change_map_traffic_cb, (void *)(!traffic_enabled));
		else
			elm_ctxpopup_item_append(ad->ctxpopup, "Turn on Traffic", NULL, change_map_traffic_cb, (void *)(!traffic_enabled));
	}

	bool public_transit_enabled = false;
	ret = maps_view_get_public_transit_enabled(ad->map_view_handle, &public_transit_enabled);
	if (ret == MAPS_ERROR_NONE) {
		/* append item to ctxpopup about transit */
		if (public_transit_enabled)
			elm_ctxpopup_item_append(ad->ctxpopup, "Turn off Public transit", NULL, change_map_public_transit_cb, (void *)(!public_transit_enabled));
		else
			elm_ctxpopup_item_append(ad->ctxpopup, "Turn on Public transit", NULL, change_map_public_transit_cb, (void *)(!public_transit_enabled));
	}
	elm_ctxpopup_direction_priority_set(ad->ctxpopup, ELM_CTXPOPUP_DIRECTION_UP, ELM_CTXPOPUP_DIRECTION_UNKNOWN, ELM_CTXPOPUP_DIRECTION_UNKNOWN, ELM_CTXPOPUP_DIRECTION_UNKNOWN);

	move_more_ctxpopup(ad->ctxpopup);
	evas_object_show(ad->ctxpopup);
}
