/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "main_view.h"
#include "search_view.h"
#include "route_view.h"
#include "util.h"

/**
 * @brief Create a button to show the result.
 * @param[in] ad The application data
 */
void create_address_button(appdata_s *ad)
{
	char position[30] = {0,};
	snprintf(position, 30, "%f, %f", ad->tmp_place->location.latitude, ad->tmp_place->location.longitude);

	/*
	 * Box
	 * Create and initialize elm_box.
	 */
	Evas_Object *address_box = elm_box_add(ad->main_view_layout);
	evas_object_size_hint_align_set(address_box, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_size_hint_weight_set(address_box, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_show(address_box);
	elm_object_part_content_set(ad->main_view_layout, "address", address_box);

	/*
	 * Button
	 * Create and initialize elm_button.
	 */
	ad->main_address_btn = elm_button_add(address_box);
	elm_object_text_set(ad->main_address_btn, position);
	elm_object_disabled_set(ad->main_address_btn, EINA_TRUE);
	evas_object_size_hint_align_set(ad->main_address_btn, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_size_hint_weight_set(ad->main_address_btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_show(ad->main_address_btn);
	elm_box_pack_end(address_box, ad->main_address_btn);
}

/**
 * @brief Function will be operated when received the result of search.
 * @param[in] result The error of result
 * @param[in] request_id The request id of search
 * @param[in] index The index of result
 * @param[in] total The total count of result
 * @param[in] coord The coord handle of result
 * @param[in] user_data The user data
 */
static bool __search_name_result_cb(maps_error_e result, int request_id, int index, int total, maps_coordinates_h coord, void *user_data)
{
	dlog_print(DLOG_DEBUG, LOG_TAG, "__search_name_result_cb : [%d/%d]", index, total);

	appdata_s *ad = (appdata_s *)user_data;
	double latitude = 0.0, longitude = 0.0;
	int ret = MAPS_ERROR_NONE;

	if (result != MAPS_ERROR_NONE)
		return false;

	ret = maps_coordinates_get_latitude_longitude(coord, &latitude, &longitude);
	if (ret != MAPS_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "maps_coordinates_get_latitude_longitude : fail [%d]", ret);
		return false;
	}
	char keyword[PATH_MAX] = {0,};
	strcpy(keyword, ad->tmp_place->place_name);
	init_view(ad);
	elm_entry_input_panel_hide(ad->main_entry);

	ad->tmp_place = (place_info_s *) malloc(sizeof(place_info_s));
	ad->tmp_place->location.latitude = latitude;
	ad->tmp_place->location.longitude = longitude;

	elm_entry_entry_set(ad->main_entry, keyword);

	maps_view_set_center(ad->map_view_handle, coord);
	add_marker(ad->map_view_handle, ad->tmp_place, true);

	create_address_button(ad);

	return false;
}

/**
 * @brief Function will be operated when the search of keyboard is pressed.
 * @param[in] data The data to be passed to the callback function
 * @param[in] obj The Evas object handle to be passed to the callback function
 * @param[in] part The part of content
 */
static void __search_pressed_cb(void *data, Evas_Object *obj, void *event_info)
{
	appdata_s *ad = (appdata_s *)data;
	maps_preference_h preference = NULL;
	int ret = 0;
	int request_id = 0;

	if (elm_entry_entry_get(obj) == NULL)
		return;
	else {
		ret = maps_preference_create(&preference);
		if (ret != MAPS_ERROR_NONE)
			return;

		if (!ad->tmp_place)
			ad->tmp_place = (place_info_s *) malloc(sizeof(place_info_s));
		strcpy(ad->tmp_place->place_name, elm_entry_entry_get(obj));

		dlog_print(DLOG_DEBUG, LOG_TAG, "__search_btn_pressed_cb : [%s]", elm_entry_entry_get(obj));
		ret = maps_service_geocode(ad->maps_service_handle,
										elm_entry_entry_get(obj),
										preference,
										__search_name_result_cb,
										ad,
										&request_id);
		if (ret != MAPS_ERROR_NONE)
			return;

		ret = maps_preference_destroy(preference);
		if (ret != MAPS_ERROR_NONE)
			return;
	}
}

/**
 * @brief Create a content of genlist
 * @param[in] data The data to be passed to the callback function
 * @param[in] obj The Evas object handle to be passed to the callback function
 * @param[in] part The part of content
 */
static Evas_Object *__search_gl_content_cb(void *data, Evas_Object *obj, const char *part)
{
	appdata_s *ad = evas_object_data_get(obj, "app_data");

	if (!strcmp(part, "elm.swallow.content")) {
		/* create an editfield */
		Evas_Object *editfield = create_editfield(obj);
		eext_entry_selection_back_event_allow_set(editfield, EINA_TRUE);

		ad->main_entry = elm_object_part_content_get(editfield, "elm.swallow.content");
		evas_object_size_hint_weight_set(ad->main_entry, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
		evas_object_size_hint_align_set(ad->main_entry, EVAS_HINT_FILL, EVAS_HINT_FILL);
		elm_object_part_text_set(ad->main_entry, "elm.guide", "Search place");

		evas_object_smart_callback_add(ad->main_entry, "activated", __search_pressed_cb, ad);

		return editfield;
	}
	return NULL;
}

/**
 * @brief Create a genlist for search field.
 * @param[in] ad The application data
 */
static void __create_genlist_layout(appdata_s *ad)
{
	/*
	 * Genlist
	 * Create and initialize elm_genlist.
	 */
	Evas_Object *genlist = elm_genlist_add(ad->main_view_layout);
	evas_object_size_hint_weight_set(genlist, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(genlist, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_genlist_mode_set(genlist, ELM_LIST_COMPRESS);
	evas_object_show(genlist);

	evas_object_data_set(genlist, "app_data", ad);

	Elm_Genlist_Item_Class *search_itc = elm_genlist_item_class_new();
	search_itc->item_style = "full";
	search_itc->func.text_get = NULL;
	search_itc->func.content_get = __search_gl_content_cb;
	search_itc->func.del = NULL;

	/* append an item to genlist */
	elm_genlist_item_append(genlist, search_itc, NULL, NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);

	elm_genlist_item_class_free(search_itc);

	elm_genlist_realized_items_update(genlist);
	elm_object_part_content_set(ad->main_view_layout, "searchbar", genlist);
}

/**
 * @brief Function will be operated when registered event is triggered.
 * @param[in] type The event type on maps view
 * @param[in] event_data The handle of event data
 * @param[in] user_data The user data
 */
static void main_view_event_gesture_cb(const maps_view_event_type_e type, maps_view_event_data_h event_data, void *user_data)
{
	dlog_print(DLOG_DEBUG, LOG_TAG, "main_view_event_gesture_cb : Enter");
	appdata_s *ad = (appdata_s *)user_data;
	int ret = MAPS_ERROR_NONE;

	if (type == MAPS_VIEW_EVENT_GESTURE) {
		maps_view_gesture_e gesture_type = MAPS_VIEW_GESTURE_NONE;
		maps_view_event_data_get_gesture_type(event_data, &gesture_type);

		if (gesture_type == MAPS_VIEW_GESTURE_LONG_PRESS) {
			if (!ad->is_reverse_geocode_supported) return;

			init_view(ad);

			maps_coordinates_h coord = NULL;
			int x = 0, y = 0;
			double latitude = 0.0, longitude = 0.0;

			cancel_revgeocode_request(ad);

			ret = maps_view_event_data_get_position(event_data, &x, &y);
			if (ret != MAPS_ERROR_NONE) {
				dlog_print(DLOG_ERROR, LOG_TAG, "maps_view_event_data_get_position : fail [%d]", ret);
				return;
			}
			ret = maps_view_screen_to_geolocation(ad->map_view_handle, x, y, &coord);
			if (ret != MAPS_ERROR_NONE) {
				dlog_print(DLOG_ERROR, LOG_TAG, "map_view_screen_to_geolocation : fail [%d]", ret);
				return;
			}
			ret = maps_coordinates_get_latitude_longitude(coord, &latitude, &longitude);
			if (ret != MAPS_ERROR_NONE) {
				dlog_print(DLOG_ERROR, LOG_TAG, "maps_coordinates_get_latitude_longitude : fail [%d]", ret);
				return;
			}

			ad->tmp_place = (place_info_s *) malloc(sizeof(place_info_s));
			ad->tmp_place->location.latitude = latitude;
			ad->tmp_place->location.longitude = longitude;

			add_marker(ad->map_view_handle, ad->tmp_place, true);

			maps_coordinates_destroy(coord);
			request_revgeocode(ad, latitude, longitude);
		}
	}
}

/**
 * @brief Create a maps view and set the initial value.
 * @param[in] ad The application data
 */
static Evas_Object *__create_map_layout(appdata_s *ad)
{
	char edj_path[PATH_MAX] = {0, };

	/*
	 * Map Layout
	 * Create and initialize elm_layout.
	 */
	Evas_Object *map_layout = elm_layout_add(ad->nf);
	app_get_resource(MAIN_VIEW_EDJ_FILE, edj_path, (int)PATH_MAX);
	elm_layout_file_set(map_layout, edj_path, "map_object");

	evas_object_size_hint_weight_set(map_layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(map_layout, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_show(map_layout);

	/*
	 * Evas Object
	 * Create and initialize elm_layout.
	 */
	Evas_Object *map_image_layout = elm_layout_add(map_layout);
	elm_layout_content_set(map_layout, "content", map_image_layout);

	/* Create Maps View handle */
	int error = maps_view_create(ad->maps_service_handle, map_image_layout, &(ad->map_view_handle));
	if (error == MAPS_ERROR_NONE) {
		/* Set a zoom level */
		maps_view_set_zoom_level(ad->map_view_handle, 15);

		/* Create a coordinate and set it to center */
		maps_coordinates_h maps_coord;
		maps_coordinates_create(DEFAULT_LAT, DEFAULT_LON, &maps_coord);
		maps_view_set_center(ad->map_view_handle, maps_coord);
		maps_coordinates_destroy(maps_coord);
	}
	return map_layout;
}

/**
 * @brief Initialize the map view and resource.
 * @param[in] ad The application data
 */
void init_view(appdata_s *ad)
{
	cancel_revgeocode_request(ad);
	cancel_search_place(ad);
	cancel_route_request(ad);

	maps_view_remove_all_objects(ad->map_view_handle);

	/* remove the resource of main view */
	if (ad->tmp_place) {
		Evas_Object *address_box = elm_object_part_content_get(ad->main_view_layout, "address");
		evas_object_del(address_box);
		elm_entry_entry_set(ad->main_entry, "");
		elm_object_part_text_set(ad->main_entry, "elm.guide", "Search place");
		evas_object_del(ad->main_address_btn);
		elm_object_part_content_unset(ad->main_view_layout, "address");

		free(ad->tmp_place);
		ad->tmp_place = NULL;
	}

	/* remove the resource of search view */
	if (ad->poi_places[0] != NULL)
		remove_all_pois(ad);

	/* remove the resource of route view */
	if (ad->start_place != NULL || ad->destination != NULL) {
		Evas_Object *info_box = elm_object_part_content_get(ad->route_view_layout, "info");
		evas_object_del(info_box);
		elm_entry_entry_set(ad->route_from_entry, "");
		elm_object_part_text_set(ad->route_from_entry, "elm.guide", "Long press in Map");
		elm_entry_entry_set(ad->route_to_entry, "");
		elm_object_part_text_set(ad->route_to_entry, "elm.guide", "Long press in Map");

		if (ad->start_place != NULL)
			free(ad->start_place);
		if (ad->destination != NULL)
			free(ad->destination);
		if (ad->route != NULL)
			free(ad->route);
		ad->start_place = NULL;
		ad->destination = NULL;
		ad->route = NULL;
	}
}

/**
 * @brief Creates the main view.
 * @param[in] data The data to be passed to the callback function
 * @param[in] obj The Evas object handle to be passed to the callback function
 * @param[in] event_info The system event information
 */
void create_main_view(void *data, Evas_Object *obj, void *event_info)
{
	dlog_print(DLOG_DEBUG, LOG_TAG, "create_main_view : Enter");
	appdata_s *ad = (appdata_s *)data;

	if (ad->prev_view == MAIN_VIEW)
		return;

	/* init map view and resource */
	init_view(ad);
	ad->prev_view = MAIN_VIEW;

	char edj_path[PATH_MAX] = {0, };
	app_get_resource(MAIN_VIEW_EDJ_FILE, edj_path, (int)PATH_MAX);

	/*
	 * Main View Layout
	 * Create and initialize elm_layout.
	 */
	ad->main_view_layout = elm_layout_add(ad->nf);
	elm_layout_file_set(ad->main_view_layout, edj_path, "map-view");
	evas_object_size_hint_weight_set(ad->main_view_layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(ad->main_view_layout, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_show(ad->main_view_layout);

	/* create a genlist */
	__create_genlist_layout(ad);

	/* create a map layout */
	if (ad->main_map_layout == NULL)
		ad->main_map_layout = __create_map_layout(ad);
	elm_object_part_content_set(ad->main_view_layout, "map", ad->main_map_layout);
	maps_view_unset_event_cb(ad->map_view_handle, MAPS_VIEW_EVENT_GESTURE);
	maps_view_unset_event_cb(ad->map_view_handle, MAPS_VIEW_EVENT_OBJECT);
	maps_view_set_event_cb(ad->map_view_handle, MAPS_VIEW_EVENT_GESTURE, main_view_event_gesture_cb, ad);

	elm_object_item_content_set(ad->nf_it, ad->main_view_layout);

	/*
	 * More Button
	 * Create and initialize elm_button.
	 */
	Evas_Object *more_btn = elm_button_add(ad->nf);
	elm_object_style_set(more_btn, "naviframe/more/default");
	elm_object_focus_allow_set(more_btn, EINA_FALSE);
	evas_object_smart_callback_add(more_btn, "clicked", create_ctxpopup_cb, ad);
	elm_object_item_part_content_set(ad->nf_it, "toolbar_more_btn", more_btn);
}
