/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "search_view.h"
#include "util.h"
#include "place.h"

int __place_request_id = REQ_ID_IDLE;

/**
 * @brief Get the category id of HERE.
 * @param[in] category The place type
 */
char *__get_here_category_id(place_type category)
{
	switch (category) {
	case food_drink:			return strdup("eat-drink");
	case transport:				return strdup("transport");
	case accommodation:			return strdup("accommodation");
	case shopping_finance:		return strdup("shopping");
	case leisure_entertainment:	return strdup("leisure-outdoor");
	default:					break;
	}

	return strdup("");
}

/**
 * @brief Function will be operated when received the result of search place.
 * @param[in] result The error of result
 * @param[in] request_id The request id of search place
 * @param[in] index The index of result
 * @param[in] length The length of result
 * @param[in] place The place handle of result
 * @param[in] user_data The user data
 */
static bool __maps_service_search_place_cb(maps_error_e error, int request_id , int index, int length , maps_place_h place , void *user_data)
{
	dlog_print(DLOG_DEBUG, LOG_TAG, "Place result >> index [%d]/ length [%d]", index, length);

	appdata_s *ad = (appdata_s *)user_data;
	maps_coordinates_h coordinates = NULL;
	double cur_lat = 0.0, cur_lon = 0.0;
	char *name = NULL;

	if (error != MAPS_ERROR_NONE || !place) {
		ad->total_poi = index;
		show_category_result(ad);
		maps_place_destroy(place);
		return false;
	}

	maps_view_get_center(ad->map_view_handle, &coordinates);
	maps_coordinates_get_latitude_longitude(coordinates, &cur_lat, &cur_lon);
	maps_coordinates_destroy(coordinates);
	coordinates = NULL;

	ad->poi_places[index] = (place_info_s *) malloc(sizeof(place_info_s));

	/* Place Name */
	maps_place_get_name(place , &name);
	strcpy(ad->poi_places[index]->place_name, name);

	/* Place Location */
	maps_place_get_location(place,  &coordinates);
	maps_coordinates_get_latitude_longitude(coordinates, &(ad->poi_places[index]->location.latitude), &(ad->poi_places[index]->location.longitude));
	maps_coordinates_destroy(coordinates);

	/* Distance Calculation */
	location_manager_get_distance(cur_lat, cur_lon, ad->poi_places[index]->location.latitude, ad->poi_places[index]->location.longitude, &(ad->poi_places[index]->distance));
	ad->poi_places[index]->distance = ad->poi_places[index]->distance * 0.001;

	if (index == (length-1)) {
		ad->total_poi = length;
		show_category_result(ad);
	}

	/* Release the place result */
	maps_place_destroy(place);

	__place_request_id = REQ_ID_IDLE;
	return true;
}

/**
 * @brief Request the search place.
 * @param[in] ad The application data
 * @param[in] search_keyword The keyword
 * @param[in] category_type The place type
 */
int request_search_place(appdata_s *ad, char *search_keyword, place_type category_type)
{
	int ret = 0;
	int request_id = 0;

	maps_place_filter_h filter = NULL;
	maps_preference_h preference = NULL;
	maps_place_category_h category = NULL;
	maps_coordinates_h coords = NULL;

	maps_preference_create(&preference);	/* Create Maps Preference */
	maps_place_filter_create(&filter);		/* Create Maps Place Filter */

	maps_place_category_create(&category);	/* Create place_category */
	maps_place_category_set_id(category, __get_here_category_id(category_type));	/* Set Category Name for place search */

	ret = maps_place_filter_set_category(filter, category);	/* Set place_category to filter */
	if (ret != MAPS_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "maps_place_filter_set error [%d]", ret);
		goto EXIT;
	}

	if (search_keyword != NULL) {
		dlog_print(DLOG_DEBUG, LOG_TAG, "poi search keyword : %s", search_keyword);
		ret = maps_place_filter_set_keyword(filter, search_keyword);	/* Keyword Search. No POI category search */
		if (ret != MAPS_ERROR_NONE)
			dlog_print(DLOG_ERROR, LOG_TAG, "maps_place_filter_set error [%d]", ret);
	}

	maps_preference_set_max_results(preference, MAX_RESULT_OF_SEARCH);	/* Set Max Results Preference */

	ret = maps_view_get_center(ad->map_view_handle, &coords);
	if (ret != MAPS_ERROR_NONE) {
		dlog_print(DLOG_DEBUG, LOG_TAG, "map_view_get_center : fail [%d]", ret);
		goto EXIT;
	}

	/* Places Search */
	ret = maps_service_search_place(ad->maps_service_handle,
							coords,
							RADIUS_OF_SEARCH,
							filter,
							preference,
							__maps_service_search_place_cb,
							(void *) ad,
							&request_id);

	if (ret != MAPS_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to poi_service_search.");
		__place_request_id = REQ_ID_IDLE;
		goto EXIT;
	} else {
		__place_request_id = request_id;
		dlog_print(DLOG_ERROR, LOG_TAG, "request_id : %d", __place_request_id);
	}

EXIT:
	if (coords)
		maps_coordinates_destroy(coords);
	if (category)
		maps_place_category_destroy(category);
	if (filter)
		maps_place_filter_destroy(filter);
	maps_preference_destroy(preference);

	return ret;
}

/**
 * @brief Cancel the request of search place.
 * @param[in] ad The application data
 */
bool cancel_search_place(appdata_s *ad)
{
	if (__place_request_id != REQ_ID_IDLE) {
		if (maps_service_cancel_request(ad->maps_service_handle, __place_request_id) == MAPS_ERROR_NONE) {
			dlog_print(DLOG_ERROR, LOG_TAG, "Place Request Cancelled");
			__place_request_id = REQ_ID_IDLE;
			return true;
		} else {
			dlog_print(DLOG_ERROR, LOG_TAG, "Place Cancel Request failed");
		}
	} else {
		dlog_print(DLOG_ERROR, LOG_TAG, "No Place Request ongoing to be cancelled");
	}

	return false;
}
