/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "revgeocode.h"
#include "main_view.h"
#include "util.h"

int __geocode_request_id = REQ_ID_IDLE;

/**
 * @brief Function will be operated when received the result of reverse geocode.
 * @param[in] result The error of result
 * @param[in] request_id The request id of reverse geocode
 * @param[in] index The index of result
 * @param[in] total The total count of result
 * @param[in] address The address handle of result
 * @param[in] user_data The user data
 */
static void reverse_geocode_cb(maps_error_e result, int request_id, int index, int total, maps_address_h address, void *user_data)
{
	dlog_print(DLOG_DEBUG, LOG_TAG, "reverse_geocode_cb : Enter");
	appdata_s *ad = (appdata_s *)user_data;

	if (result != MAPS_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Invalid Reverse Geocode Result");
		return;
	}

	char resultText[1024] = {0, };

	char *street = NULL;
	maps_address_get_street(address, &street);
	if (street != NULL) {
		strcat(resultText, street);
		free(street);
		street = NULL;
	} else {
		dlog_print(DLOG_ERROR, LOG_TAG, "Street is NULL");
	}

	char *district = NULL;
	maps_address_get_district(address, &district);
	if (district != NULL) {
		if (strlen(resultText) > 0)
			strcat(resultText, ", ");
		strcat(resultText, district);
		free(district);
		district = NULL;
	} else {
		dlog_print(DLOG_ERROR, LOG_TAG, "District is NULL");
	}

	char *city = NULL;
	maps_address_get_city(address, &city);
	if (city != NULL) {
		if (strlen(resultText) > 0)
			strcat(resultText, ", ");
		strcat(resultText, city);
		free(city);
		city = NULL;
	} else {
		dlog_print(DLOG_ERROR, LOG_TAG, "City is NULL");
	}

	char *state = NULL;
	maps_address_get_state(address, &state);
	if (state != NULL) {
		if (strlen(resultText) > 0)
			strcat(resultText, ", ");
		strcat(resultText, state);
		free(state);
		state = NULL;
	} else {
		dlog_print(DLOG_ERROR, LOG_TAG, "State is NULL");
	}

	char *country = NULL;
	maps_address_get_country(address, &country);
	if (country != NULL) {
		if (strlen(resultText) > 0)
			strcat(resultText, ", ");
		strcat(resultText, country);
		free(country);
		country = NULL;
	} else {
		dlog_print(DLOG_ERROR, LOG_TAG, "Country is NULL");
	}

	char *country_code = NULL;
	maps_address_get_country_code(address, &country_code);
	if (country_code != NULL) {
		if (strlen(resultText) > 0)
			strcat(resultText, ", ");
		strcat(resultText, country_code);
		free(country_code);
		country_code = NULL;
	} else {
		dlog_print(DLOG_ERROR, LOG_TAG, "Country Code is NULL");
	}

	char *postal_code = NULL;
	maps_address_get_postal_code(address, &postal_code);
	if (postal_code != NULL) {
		if (strlen(resultText) > 0)
			strcat(resultText, ", ");
		strcat(resultText, postal_code);
		free(postal_code);
		postal_code = NULL;
	} else {
		dlog_print(DLOG_ERROR, LOG_TAG, "Postal Code is NULL");
	}

	if (strlen(resultText) > 0) {
		strcpy(ad->tmp_place->place_name, resultText);
		create_address_button(ad);
	}

	elm_object_part_text_set(ad->main_entry, "elm.guide", ad->tmp_place->place_name);
	dlog_print(DLOG_DEBUG, LOG_TAG, "address: %s", ad->tmp_place->place_name);
	maps_address_destroy(address);

	__geocode_request_id = REQ_ID_IDLE;

	elm_entry_input_panel_hide(ad->main_entry);
}

/**
 * @brief Request the reverse geocode
 * param[in] ad The application data
 * @param[in] latitude The value of latitude
 * @param[in] longitude The value of longitude
 */
int request_revgeocode(appdata_s *ad, double latitude, double longitude)
{
	dlog_print(DLOG_DEBUG, LOG_TAG, "request_revgeocode : Enter");

	int request_id = -1;

	maps_preference_h preference = NULL;
	maps_preference_create(&preference);

	int error = maps_service_reverse_geocode(ad->maps_service_handle,			/* maps service handle */
													latitude,					/* center location */
													longitude,					/* search radius from center location */
													preference,					/* reverse geocode preference */
													reverse_geocode_cb,			/* callback */
													(void *)ad,					/* user_data */
													&request_id);				/* request_id */

	if (error != MAPS_ERROR_NONE)
		__geocode_request_id = REQ_ID_IDLE;
	else
		__geocode_request_id = request_id;

	maps_preference_destroy(preference);

	return error;
}

/**
 * @brief Cancel the request of reverse geocode.
 * @param[in] ad The application data
 */
bool cancel_revgeocode_request(appdata_s *ad)
{
	if (__geocode_request_id != REQ_ID_IDLE) {
		if (maps_service_cancel_request(ad->maps_service_handle, __geocode_request_id) == MAPS_ERROR_NONE) {
			dlog_print(DLOG_ERROR, LOG_TAG, "Reverse Geocode Request Cancelled");
			__geocode_request_id = REQ_ID_IDLE;
			return true;
		} else {
			dlog_print(DLOG_ERROR, LOG_TAG, "Reverse Geocode Cancel Request failed");
		}
	} else {
		dlog_print(DLOG_ERROR, LOG_TAG, "No Reverse Geocode Request ongoing to be cancelled");
	}

	return false;
}
