/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "route.h"
#include "route_view.h"
#include "main_view.h"

int __route_request_id = REQ_ID_IDLE;

/**
 * @brief Function will be operated when received the result of retrieving route.
 * @param[in] index The index of result
 * @param[in] total The total count of result
 * @param[in] coordinates The coordinates handle of result
 * @param[in] user_data The user data
 */
static bool __maps_route_path_cb(int index, int total, maps_coordinates_h coordinates, void *user_data)
{
	appdata_s *ad = (appdata_s *)user_data;

	double lat, lon;
	maps_coordinates_get_latitude_longitude(coordinates, &lat, &lon);
	maps_coordinates_destroy(coordinates);

	(ad->route->path[index]).latitude = lat;
	(ad->route->path[index]).longitude = lon;
	ad->route->path_count = index + 1;
	return true;
}

/**
 * @brief Parse the result of search route.
 * @param[in] route The route handle
 * @param[in] index The index
 * @param[in] total The total count
 * @param[in] ad The application data
 */
static void __parse_route_data(maps_route_h route, int index, int total, appdata_s *ad)
{
	maps_distance_unit_e dist_unit;
	double distance = 0;
	long duration = 0;

	/* Route - Total Distance */
	maps_route_get_total_distance(route, &distance);
	/* Route - Distance Unit */
	maps_route_get_distance_unit(route, &dist_unit);

	/* Changing everything to KM (by default) */
	if (dist_unit == MAPS_DISTANCE_UNIT_M)
		distance = distance / 1000;	/* convert the distance into km */
	dlog_print(DLOG_DEBUG, LOG_TAG, "distance : %.5f km", distance);

	/* Route - Total Duration */
	maps_route_get_total_duration(route, &duration);
	dlog_print(DLOG_DEBUG, LOG_TAG, "duration : %ld sec", duration);
	duration = (duration + 30) / 60;	/*converting duration to minutes */

	/* Route - Origin, destination */
	maps_coordinates_h coord;
	double origin_lat, origin_lon, dest_lat, dest_lon;

	maps_route_get_origin(route, &coord);
	maps_coordinates_get_latitude_longitude(coord, &origin_lat, &origin_lon);
	maps_coordinates_destroy(coord);

	maps_route_get_destination(route, &coord);
	maps_coordinates_get_latitude_longitude(coord, &dest_lat, &dest_lon);
	maps_coordinates_destroy(coord);

	dlog_print(DLOG_ERROR, LOG_TAG, "origin=%f,%f  destination=%f,%f", origin_lat, origin_lon, dest_lat, dest_lon);
	ad->route = (route_info_s *)malloc(sizeof(route_info_s));
	ad->route->distance = distance;
	ad->route->duration = duration;

	/* Check if maneuver path data is supported */
	bool supported = false;
	int error = maps_service_provider_is_data_supported(ad->maps_service_handle,
											MAPS_ROUTE_SEGMENTS_PATH, &supported);
	const bool is_route_segment_path_supported = (error == MAPS_ERROR_NONE) ? supported : false;

	/* Check if maneuver sements data is supported */
	error = maps_service_provider_is_data_supported(ad->maps_service_handle,
											MAPS_ROUTE_SEGMENTS_MANEUVERS, &supported);
	const bool is_route_segment_maneuvers_supported = (error == MAPS_ERROR_NONE) ? supported : false;

	if ((is_route_segment_path_supported) && (is_route_segment_maneuvers_supported)) {
		/* Allow segment maneuvers and path usage */
		maps_route_foreach_path(route, __maps_route_path_cb, (void *)ad);
	}
}

/**
 * @brief Function will be operated when received the result of search route.
 * @param[in] result The error of result
 * @param[in] request_id The request id of search route
 * @param[in] index The index of result
 * @param[in] total The total count of result
 * @param[in] route The route handle of result
 * @param[in] user_data The user data
 */
static bool __maps_service_search_route_cb(maps_error_e error, int request_id, int index, int total, maps_route_h route, void *user_data)
{
	appdata_s *ad = (appdata_s *)user_data;

	if (!route) {
		dlog_print(DLOG_ERROR, LOG_TAG, "route handle is null");
		return false;
	}

	__parse_route_data(route, index, total, ad);

	show_route_in_map(ad);

	maps_route_destroy(route);

	__route_request_id = REQ_ID_IDLE;

	return false;
}

/**
 * @brief Request the search route.
 * @param[in] ad The application data
 * @param[in] src_lat The latitude of start position
 * @param[in] src_lon The longitude of start position
 * @param[in] src_lat The latitude of destination
 * @param[in] src_lon The longitude of destination
 */
int request_route(appdata_s *ad, double src_lat, double src_lon, double dest_lat, double dest_lon)
{
	dlog_print(DLOG_ERROR, LOG_TAG, "request_route origin=%f,%f destination=%f,%f", src_lat, src_lon, dest_lat, dest_lon);
	int request_id = -1;
	int ret = MAPS_ERROR_NONE;

	if (ad->route)
		free(ad->route);
	ad->route = NULL;

	maps_coordinates_h origin = NULL;
	maps_coordinates_h destination = NULL;

	/* Origin Coordinates */
	maps_coordinates_create(src_lat, src_lon, &origin);
	/* Destination Coordinates */
	maps_coordinates_create(dest_lat, dest_lon, &destination);

	/* Maps Preference */
	maps_preference_h preference = NULL;
	maps_preference_create(&preference);

	/* Set Route Transport mode Preference */
	/* Transport Mode - Car */
	maps_preference_set_route_transport_mode(preference, MAPS_ROUTE_TRANSPORT_MODE_CAR);

	/* Route Search */
	ret = maps_service_search_route(ad->maps_service_handle,
											origin,
											destination,
											preference,
											__maps_service_search_route_cb,
											ad,
											&request_id);

	if (ret == MAPS_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "request_id : %d", request_id);
		__route_request_id = request_id;
	} else {
		dlog_print(DLOG_ERROR, LOG_TAG, "Route Service Request Failed ::  [%d]", ret);
		__route_request_id = REQ_ID_IDLE;
	}

	maps_coordinates_destroy(origin);
	maps_coordinates_destroy(destination);
	maps_preference_destroy(preference);

	return ret;
}

/**
 * @brief Cancel the request of search route.
 * @param[in] ad The application data
 */
bool cancel_route_request(appdata_s *ad)
{
	if (__route_request_id != REQ_ID_IDLE) {
		if (maps_service_cancel_request(ad->maps_service_handle, __route_request_id) == MAPS_ERROR_NONE) {
			dlog_print(DLOG_ERROR, LOG_TAG, "Route Request Cancelled");
			__route_request_id = REQ_ID_IDLE;
			return true;
		} else {
			dlog_print(DLOG_ERROR, LOG_TAG, "Route Cancel Request failed");
		}
	} else {
		dlog_print(DLOG_ERROR, LOG_TAG, "No Route Request ongoing to be cancelled");
	}

	return false;
}
