/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "main_view.h"
#include "util.h"
#include "search_view.h"
#include "route.h"
#include "route_view.h"

/**
 * @brief Create a button to display the information.
 * @param[in] ad The application data
 */
static void create_info_button(appdata_s *ad)
{
	dlog_print(DLOG_ERROR, LOG_TAG, "create_info_button : Enter");
	char route_info[30] = {0,};
	snprintf(route_info, 30, "%.2lf km / %d min by car", ad->route->distance, ad->route->duration);

	Evas_Object *info_box = elm_box_add(ad->route_view_layout);
	evas_object_size_hint_align_set(info_box, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_size_hint_weight_set(info_box, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_show(info_box);
	elm_object_part_content_set(ad->route_view_layout, "info", info_box);

	ad->route_info_btn = elm_button_add(info_box);
	elm_object_text_set(ad->route_info_btn, route_info);
	elm_object_disabled_set(ad->route_info_btn, EINA_TRUE);
	evas_object_size_hint_align_set(ad->route_info_btn, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_size_hint_weight_set(ad->route_info_btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_show(ad->route_info_btn);
	elm_box_pack_end(info_box, ad->route_info_btn);
}

/**
 * @brief Show the route.
 * @param[in] ad The application data
 */
void show_route_in_map(appdata_s *ad)
{
	double lat = 0.0, lon = 0.0;

	maps_coordinates_h coord;
	maps_coordinates_list_h coord_list;
	maps_coordinates_list_create(&coord_list);

	maps_coordinates_create(ad->start_place->location.latitude, ad->start_place->location.longitude, &coord);
	maps_coordinates_list_append(coord_list, coord);

	for (int i = 0; i < ad->route->path_count && i < MAX_ROUTE_PATH; i++) {
		lat = ad->route->path[i].latitude;
		lon = ad->route->path[i].longitude;
		dlog_print(DLOG_ERROR, LOG_TAG, "Maneuver[%d] - [%f,%f]", i, lat, lon);
		maps_coordinates_create(lat, lon, &coord);
		maps_coordinates_list_append(coord_list, coord);
	}

	maps_coordinates_create(ad->destination->location.latitude, ad->destination->location.longitude, &coord);
	maps_coordinates_list_append(coord_list, coord);

	maps_view_object_h polyline;
	maps_view_object_create_polyline(coord_list, 255, 0, 0, 255, 3, &polyline);
	maps_view_add_object(ad->map_view_handle, polyline);

	create_info_button(ad);
}

/**
 * @brief Function will be operated when registered event is triggered.
 * @param[in] type The event type on maps view
 * @param[in] event_data The handle of event data
 * @param[in] user_data The user data
 */
static void route_view_event_gesture_cb(const maps_view_event_type_e type, maps_view_event_data_h event_data, void *user_data)
{
	dlog_print(DLOG_ERROR, LOG_TAG, "route_view_event_gesture_cb : Enter");

	appdata_s *ad = (appdata_s *)user_data;
	int ret = MAPS_ERROR_NONE;

	if (type == MAPS_VIEW_EVENT_GESTURE) {
		maps_view_gesture_e gesture_type = MAPS_VIEW_GESTURE_NONE;
		maps_view_event_data_get_gesture_type(event_data, &gesture_type);

		if (gesture_type == MAPS_VIEW_GESTURE_LONG_PRESS) {
			dlog_print(DLOG_ERROR, LOG_TAG, "MAP_GESTURE_LONG_PRESS");

			if (!ad->is_reverse_geocode_supported) return;

			maps_coordinates_h coord = NULL;
			int x = 0, y = 0;
			double latitude = 0.0, longitude = 0.0;

			ret = maps_view_event_data_get_position(event_data, &x, &y);
			if (ret != MAPS_ERROR_NONE) {
				dlog_print(DLOG_ERROR, LOG_TAG, "map_event_data_get_xy : fail [%d]", ret);
				return;
			}
			ret = maps_view_screen_to_geolocation(ad->map_view_handle, x, y, &coord);
			if (ret != MAPS_ERROR_NONE) {
				dlog_print(DLOG_ERROR, LOG_TAG, "map_view_screen_to_geolocation : fail [%d]", ret);
				return;
			}
			ret = maps_coordinates_get_latitude_longitude(coord, &latitude, &longitude);
			if (ret != MAPS_ERROR_NONE) {
				dlog_print(DLOG_ERROR, LOG_TAG, "maps_coordinates_get_latitude_longitude : fail [%d]", ret);
				return;
			}

			maps_coordinates_destroy(coord);

			if (strlen(elm_entry_entry_get(ad->route_from_entry)) < 1) {
				if (ad->start_place)
					free(ad->start_place);
				ad->start_place = NULL;

				ad->start_place = (place_info_s *) malloc(sizeof(place_info_s));
				ad->start_place->location.latitude = latitude;
				ad->start_place->location.longitude = longitude;

				snprintf(ad->start_place->place_name, sizeof(ad->start_place->place_name), "%lf, %lf", ad->start_place->location.latitude, ad->start_place->location.longitude);
				elm_entry_entry_set(ad->route_from_entry, ad->start_place->place_name);
			} else if (strlen(elm_entry_entry_get(ad->route_to_entry)) < 1) {
				dlog_print(DLOG_ERROR, LOG_TAG, "set destination");
				if (ad->destination)
					free(ad->destination);
				ad->destination = NULL;

				ad->destination = (place_info_s *) malloc(sizeof(place_info_s));
				ad->destination->location.latitude = latitude;
				ad->destination->location.longitude = longitude;

				snprintf(ad->destination->place_name, sizeof(ad->destination->place_name), "%lf, %lf", ad->destination->location.latitude, ad->destination->location.longitude);
				elm_entry_entry_set(ad->route_to_entry, ad->destination->place_name);
			} else
				return;

			if (ad->start_place)
				add_marker(ad->map_view_handle, ad->start_place, true);
			if (ad->destination)
				add_marker(ad->map_view_handle, ad->destination, false);

			if (ad->start_place && ad->destination) {
				if (ad->is_search_route_supported) {
					request_route(ad,
									ad->start_place->location.latitude,
									ad->start_place->location.longitude,
									ad->destination->location.latitude,
									ad->destination->location.longitude);
				}
			}
		}
	}
}

/**
 * @brief Create a toolbar for route view.
 * @param[in] ad The application data
 */
static Evas_Object *create_route_toolbar(appdata_s *ad)
{
	/* toolbar */
	Evas_Object *toolbar = elm_toolbar_add(ad->nf);
	elm_object_style_set(toolbar, "tabbar");
	elm_toolbar_shrink_mode_set(toolbar, ELM_TOOLBAR_SHRINK_EXPAND);
	elm_toolbar_transverse_expanded_set(toolbar, EINA_TRUE);
	elm_toolbar_select_mode_set(toolbar, ELM_OBJECT_SELECT_MODE_DEFAULT);

	elm_toolbar_item_append(toolbar, NULL, "Map", create_main_view, ad);
	elm_toolbar_item_append(toolbar, NULL, "POI", create_search_view, ad);
	elm_toolbar_item_selected_set(elm_toolbar_item_append(toolbar, NULL, "Route", create_route_view, ad), EINA_TRUE);

	elm_object_item_part_content_set(ad->nf_it, "tabbar", toolbar);

	return toolbar;
}

/**
 * @brief Create a content of genlist.
 * @param[in] data The data to be passed to the callback function
 * @param[in] obj The Evas object handle to be passed to the callback function
 * @param[in] part The part of genlist
 */
static Evas_Object *__from_gl_content_cb(void *data, Evas_Object *obj, const char *part)
{
	appdata_s *ad = evas_object_data_get(obj, "app_data");

	if (!strcmp(part, "elm.swallow.content")) {
		Evas_Object *editfield = create_editfield(obj);
		eext_entry_selection_back_event_allow_set(editfield, EINA_TRUE);

		ad->route_from_entry = elm_object_part_content_get(editfield, "elm.swallow.content");
		evas_object_size_hint_weight_set(ad->route_from_entry, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
		evas_object_size_hint_align_set(ad->route_from_entry, EVAS_HINT_FILL, EVAS_HINT_FILL);
		elm_object_part_text_set(ad->route_from_entry, "elm.guide", "From - Long press in Map");

		elm_entry_editable_set(ad->route_from_entry, EINA_FALSE);

		return editfield;
	}
	return NULL;
}

/**
 * @brief Create a content of genlist.
 * @param[in] data The data to be passed to the callback function
 * @param[in] obj The Evas object handle to be passed to the callback function
 * @param[in] part The part of genlist
 */
static Evas_Object *__to_gl_content_cb(void *data, Evas_Object *obj, const char *part)
{
	appdata_s *ad = evas_object_data_get(obj, "app_data");

	if (!strcmp(part, "elm.swallow.content")) {
		Evas_Object *editfield = create_editfield(obj);
		eext_entry_selection_back_event_allow_set(editfield, EINA_TRUE);

		ad->route_to_entry = elm_object_part_content_get(editfield, "elm.swallow.content");
		evas_object_size_hint_weight_set(ad->route_to_entry, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
		evas_object_size_hint_align_set(ad->route_to_entry, EVAS_HINT_FILL, EVAS_HINT_FILL);
		elm_object_part_text_set(ad->route_to_entry, "elm.guide", "To      - Long press in Map");

		elm_entry_editable_set(ad->route_to_entry, EINA_FALSE);

		return editfield;
	}
	return NULL;
}

/**
 * @brief Create a genlist to display the infomation.
 * @param[in] ad The application data
 */
static void __create_route_searchbox(appdata_s *ad)
{
	Evas_Object *genlist = elm_genlist_add(ad->route_view_layout);
	evas_object_size_hint_weight_set(genlist, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(genlist, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_genlist_mode_set(genlist, ELM_LIST_COMPRESS);
	evas_object_show(genlist);

	evas_object_data_set(genlist, "app_data", ad);

	Elm_Genlist_Item_Class *from_itc = elm_genlist_item_class_new();
	from_itc->item_style = "full";
	from_itc->func.text_get = NULL;
	from_itc->func.content_get = __from_gl_content_cb;
	from_itc->func.del = NULL;
	elm_genlist_item_append(genlist, from_itc, NULL, NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);

	Elm_Genlist_Item_Class *to_itc = elm_genlist_item_class_new();
	to_itc->item_style = "full";
	to_itc->func.text_get = NULL;
	to_itc->func.content_get = __to_gl_content_cb;
	to_itc->func.del = NULL;
	elm_genlist_item_append(genlist, to_itc, NULL, NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);

	elm_genlist_item_class_free(from_itc);
	elm_genlist_item_class_free(to_itc);

	elm_genlist_realized_items_update(genlist);
	elm_object_part_content_set(ad->route_view_layout, "searchbar", genlist);
}

/**
 * @brief Creates the route view.
 * @param[in] data The data to be passed to the callback function
 * @param[in] obj The Evas object handle to be passed to the callback function
 * @param[in] event_info The system event information
 */
void create_route_view(void *data, Evas_Object *obj, void *event_info)
{
	dlog_print(DLOG_DEBUG, LOG_TAG, "create_route_view : Enter");
	appdata_s *ad = (appdata_s *)data;

	if (ad->prev_view == ROUTE_VIEW)
		return;

	init_view(ad);
	ad->prev_view = ROUTE_VIEW;

	char edj_path[PATH_MAX] = {0, };
	app_get_resource(MAIN_VIEW_EDJ_FILE, edj_path, (int)PATH_MAX);

	ad->route_view_layout = elm_layout_add(ad->nf);
	elm_layout_file_set(ad->route_view_layout, edj_path, "route-view");
	evas_object_size_hint_weight_set(ad->route_view_layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(ad->route_view_layout, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_show(ad->route_view_layout);

	__create_route_searchbox(ad);

	elm_object_part_content_set(ad->route_view_layout, "map", ad->main_map_layout);
	maps_view_unset_event_cb(ad->map_view_handle, MAPS_VIEW_EVENT_GESTURE);
	maps_view_unset_event_cb(ad->map_view_handle, MAPS_VIEW_EVENT_OBJECT);
	maps_view_set_event_cb(ad->map_view_handle, MAPS_VIEW_EVENT_GESTURE, route_view_event_gesture_cb, ad);

	create_route_toolbar(ad);
	elm_object_item_content_set(ad->nf_it, ad->route_view_layout);
}
