/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "search_view.h"
#include "main_view.h"
#include "route_view.h"
#include "util.h"

const char *here_category_list_text[] = {
	"Food and Drink",
	"Transport",
	"Hotels",
	"Shopping",
	"Leisure"
};

/**
 * @brief Get a text of the place.
 * @param[in] place The place
 */
char *__get_poi_text_cb(place_info_s *place)
{
	char poi_text[1024] = {0,}, tmp[1024] = {0,};
	char *token = NULL, *next_token = NULL, *tmp_token = NULL;
	const char *seps = "&";

	if (place->distance >= 0.0)
		snprintf(tmp, sizeof(tmp), "%s (%0.2f km)", place->place_name, place->distance);
	else
		snprintf(tmp, sizeof(tmp), "%s", place->place_name);

	token = strtok_r(tmp, seps, &next_token);
	next_token = strtok_r(next_token, seps, &tmp_token);
	if (next_token != NULL)
		snprintf(poi_text, sizeof(poi_text), "%s&amp;%s", token, next_token);
	else
		snprintf(poi_text, sizeof(poi_text), "%s", tmp);

	return strdup(poi_text);
}

/**
 * @brief Function will be operated when registered event is triggered.
 * @param[in] data The data to be passed to the callback function
 * @param[in] obj The Evas object handle to be passed to the callback function
 * @param[in] event_info The system event information
 */
static void __poi_selected_cb(void *data, Evas_Object *obj, void *event_info)
{
	appdata_s *ad = evas_object_data_get(obj, "app_data");

	int selected_poi = (int) data;
	dlog_print(DLOG_DEBUG, LOG_TAG, "selected [%d] poi", selected_poi);

	update_poi_markers(ad, selected_poi);
	elm_object_text_set(ad->hoversel, __get_poi_text_cb((void *)ad->poi_places[selected_poi]));
}

/**
 * @brief Create a hoversel for search view.
 * @param[in] layout The layout for hoversel
 * @param[in] ad The application data
 */
static Evas_Object *__create_poi_hoversel(Evas_Object *layout, appdata_s *ad)
{
	Evas_Object *box = elm_box_add(layout);
	evas_object_size_hint_align_set(box, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_size_hint_weight_set(box, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_show(box);

	ad->hoversel = elm_hoversel_add(box);
	elm_hoversel_hover_parent_set(ad->hoversel, layout);
	elm_object_text_set(ad->hoversel, __get_poi_text_cb((void *)ad->poi_places[0]));

	evas_object_data_set(ad->hoversel, "app_data", ad);
	for (int i = 0; i < ad->total_poi; i++)
		elm_hoversel_item_add(ad->hoversel, __get_poi_text_cb((void *)ad->poi_places[i]), NULL, ELM_ICON_NONE, __poi_selected_cb, (void *)i);

	evas_object_size_hint_align_set(ad->hoversel, EVAS_HINT_FILL, 1.0);
	evas_object_size_hint_weight_set(ad->hoversel, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_show(ad->hoversel);
	elm_box_pack_end(box, ad->hoversel);

	return box;
}

/**
 * @brief Function will be operated when registered event is triggered.
 * @param[in] type The event type on maps view
 * @param[in] event_data The handle of event data
 * @param[in] user_data The user data
 */
static void search_view_event_object_cb(const maps_view_event_type_e type, maps_view_event_data_h event_data, void *user_data)
{
	appdata_s *ad = (appdata_s *)user_data;

	if (type == MAPS_VIEW_EVENT_OBJECT) {
		maps_view_object_h obj = NULL;
		maps_view_event_data_get_object(event_data, &obj);

		for (int i = 0; i < ad->total_poi && i < MAX_PLACES; i++) {
			if (obj == ad->poi_places[i]->marker) {
				update_poi_markers(ad, i);
				elm_object_text_set(ad->hoversel, __get_poi_text_cb((void *)ad->poi_places[i]));
				break;
			}
		}
	}
}

/**
 * @brief Show the result of search.
 * @param[in] ad The application data
 */
void show_category_result(appdata_s *ad)
{
	dlog_print(DLOG_DEBUG, LOG_TAG, "Place Result count :: [%d]", ad->total_poi);

	sort_quick_places(ad->poi_places, 0, ad->total_poi - 1);
	int i = 0;
	for (i = 0; i < ad->total_poi; i++) {
		dlog_print(DLOG_DEBUG, LOG_TAG, "place name : %s", ad->poi_places[i]->place_name);
		dlog_print(DLOG_DEBUG, LOG_TAG, "place coordinates :: %f,%f", ad->poi_places[i]->location.latitude, ad->poi_places[i]->location.longitude);
		dlog_print(DLOG_DEBUG, LOG_TAG, "place distance :: %f", ad->poi_places[i]->distance);
	}
	maps_view_remove_all_objects(ad->map_view_handle);
	ad->prev_view = SEARCH_RESULT_VIEW;

	if (ad->total_poi > 0) {
		char edj_path[PATH_MAX] = {0, };
		app_get_resource(MAIN_VIEW_EDJ_FILE, edj_path, (int)PATH_MAX);

		ad->search_result_layout = elm_layout_add(ad->nf);
		elm_layout_file_set(ad->search_result_layout, edj_path, "search-view-result");
		evas_object_size_hint_weight_set(ad->search_result_layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
		evas_object_size_hint_align_set(ad->search_result_layout, EVAS_HINT_FILL, EVAS_HINT_FILL);

		elm_object_part_content_set(ad->search_result_layout, "map", ad->main_map_layout);
		maps_view_unset_event_cb(ad->map_view_handle, MAPS_VIEW_EVENT_GESTURE);
		maps_view_unset_event_cb(ad->map_view_handle, MAPS_VIEW_EVENT_OBJECT);
		maps_view_set_event_cb(ad->map_view_handle, MAPS_VIEW_EVENT_OBJECT, search_view_event_object_cb, ad);

		add_marker(ad->map_view_handle, ad->poi_places[0], true);
		for (int i = 1; i < ad->total_poi && i < MAX_PLACES; i++)
			add_marker(ad->map_view_handle, ad->poi_places[i], false);
		maps_view_set_zoom_level(ad->map_view_handle, 15);

		Evas_Object *hoversel_layout = __create_poi_hoversel(ad->search_result_layout, ad);
		elm_object_part_content_set(ad->search_result_layout, "hoversel", hoversel_layout);

		elm_object_item_content_set(ad->nf_it, ad->search_result_layout);
	} else {
		ad->search_genlist = create_nocontent_layout(ad->search_view_layout, "No results found", NULL);
		elm_object_part_content_set(ad->search_view_layout, "list", ad->search_genlist);
	}
}

/**
 * @brief Function will be operated when registered event is triggered.
 * @param[in] data The data to be passed to the callback function
 * @param[in] obj The Evas object handle to be passed to the callback function
 * @param[in] event_info The system event information
 */
static void __done_pressed_cb(void *data, Evas_Object *obj, void *event_info)
{
	elm_entry_input_panel_hide(obj);
}

/**
 * @brief Create a search field for search view.
 * @param[in] ad The application data
 */
static Evas_Object *__create_entry_layout(appdata_s *ad)
{
	Evas_Object *searchbar_obj = elm_layout_add(ad->search_view_layout);
	evas_object_size_hint_weight_set(searchbar_obj, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(searchbar_obj, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_show(searchbar_obj);
	elm_layout_theme_set(searchbar_obj, "layout", "searchbar", "default");

	/* entry */
	Evas_Object *editfield = create_editfield(searchbar_obj);
	eext_entry_selection_back_event_allow_set(editfield, EINA_TRUE);
	evas_object_size_hint_weight_set(editfield, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(editfield, EVAS_HINT_FILL, EVAS_HINT_FILL);

	Evas_Object *search_entry = elm_object_part_content_get(editfield, "elm.swallow.content");
	evas_object_size_hint_weight_set(search_entry, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(search_entry, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_entry_single_line_set(search_entry, EINA_TRUE);

	elm_entry_input_panel_return_key_type_set(search_entry, ELM_INPUT_PANEL_RETURN_KEY_TYPE_DONE);
	elm_object_part_text_set(search_entry, "elm.guide", "Search POI");
	evas_object_show(search_entry);

	elm_object_part_content_set(searchbar_obj, "elm.swallow.content", search_entry);
	evas_object_smart_callback_add(search_entry, "activated", __done_pressed_cb, ad);

	elm_object_part_content_set(ad->search_view_layout, "searchbar", searchbar_obj);

	return search_entry;
}

/**
 * @brief Create a progressbar for search view.
 * @param[in] ad The application data
 */
static void __change_progress_view(appdata_s *ad)
{
	ad->search_genlist = elm_progressbar_add(ad->search_view_layout);
	elm_object_style_set(ad->search_genlist, "process_large");
	evas_object_size_hint_align_set(ad->search_genlist, EVAS_HINT_FILL, 0.5);
	evas_object_size_hint_weight_set(ad->search_genlist, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_progressbar_pulse(ad->search_genlist, EINA_TRUE);
	evas_object_show(ad->search_genlist);
	elm_object_focus_set(ad->search_genlist, EINA_TRUE);

	elm_object_part_content_set(ad->search_view_layout, "list", ad->search_genlist);
}

/**
 * @brief Function will be operated when registered event is triggered.
 * @param[in] data The data to be passed to the callback function
 * @param[in] obj The Evas object handle to be passed to the callback function
 * @param[in] event_info The system event information
 */
static void __category_selected_cb(void *data, Evas_Object *obj, void *event_info)
{
	appdata_s *ad = evas_object_data_get(obj, "app_data");
	int category = (int) data;

	cancel_search_place(ad);

	elm_genlist_item_selected_set((Elm_Object_Item *)event_info, EINA_FALSE);

	remove_all_pois(ad);

	if (!ad->is_search_place_supported) {
		ad->search_genlist = create_nocontent_layout(ad->search_view_layout, "Place Search not supported", NULL);
		elm_object_part_content_set(ad->search_view_layout, "list", ad->search_genlist);
		return;
	}

	__change_progress_view(ad);

	int ret = MAPS_ERROR_NONE;
	char *keyword = NULL;
	keyword = strdup(elm_entry_entry_get(ad->search_entry));
	/* POI Search */
	if (keyword != NULL && strlen(keyword) >= 1)
		ret = request_search_place(ad, keyword, category);
	else
		ret = request_search_place(ad, NULL, category);

	if (keyword != NULL)
		free(keyword);
	if (ret != MAPS_ERROR_NONE) {
		ad->search_genlist = create_nocontent_layout(ad->search_view_layout, "Place Request Failed", NULL);
		elm_object_part_content_set(ad->search_view_layout, "list", ad->search_genlist);
	}
}

/**
 * @brief Get a text for genlist.
 * @param[in] data The data to be passed to the callback function
 * @param[in] obj The Evas object handle to be passed to the callback function
 * @param[in] part The part of genlist
 */
static char *__get_here_category_text_cb(void *data, Evas_Object *obj, const char *part)
{
	int index = (int)data;
	if (!strcmp(part, "elm.text"))
		return strdup(here_category_list_text[index]);

	return NULL;
}

/**
 * @brief Create a genlist for search view.
 * @param[in] ad The application data
 */
static Evas_Object *__create_genlist_layout(appdata_s *ad)
{
	Evas_Object *genlist = elm_genlist_add(ad->search_view_layout);
	elm_list_mode_set(genlist, ELM_LIST_COMPRESS);
	evas_object_show(genlist);
	evas_object_data_set(genlist, "app_data", ad);

	/* category */
	Elm_Genlist_Item_Class *category_itc = elm_genlist_item_class_new();
	category_itc->item_style = "type1";
	category_itc->func.content_get = NULL;
	category_itc->func.text_get = __get_here_category_text_cb;

	for (int i = 0; i < NUM_OF_CATEGORY; i++)
		elm_genlist_item_append(genlist,
									category_itc,
									(void *)i,
									NULL,
									ELM_GENLIST_ITEM_NONE,
									__category_selected_cb,
									(void *)i);

	elm_genlist_item_class_free(category_itc);
	elm_object_part_content_set(ad->search_view_layout, "list", genlist);

	return genlist;
}

/**
 * @brief Create a toolbar for search view.
 * @param[in] ad The application data
 */
static Evas_Object *create_search_toolbar(appdata_s *ad)
{
	/* toolbar */
	Evas_Object *toolbar = elm_toolbar_add(ad->nf);
	elm_object_style_set(toolbar, "tabbar");
	elm_toolbar_shrink_mode_set(toolbar, ELM_TOOLBAR_SHRINK_EXPAND);
	elm_toolbar_transverse_expanded_set(toolbar, EINA_TRUE);
	elm_toolbar_select_mode_set(toolbar, ELM_OBJECT_SELECT_MODE_DEFAULT);

	elm_toolbar_item_append(toolbar, NULL, "Map", create_main_view, ad);
	elm_toolbar_item_selected_set(elm_toolbar_item_append(toolbar, NULL, "POI", create_search_view, ad), EINA_TRUE);
	elm_toolbar_item_append(toolbar, NULL, "Route", create_route_view, ad);

	elm_object_item_part_content_set(ad->nf_it, "tabbar", toolbar);

	return toolbar;
}

/**
 * @brief Creates the searh view.
 * @param[in] data The data to be passed to the callback function
 * @param[in] obj The Evas object handle to be passed to the callback function
 * @param[in] event_info The system event information
 */
void create_search_view(void *data, Evas_Object *obj, void *event_info)
{
	dlog_print(DLOG_DEBUG, LOG_TAG, "create_search_view : Enter");
	appdata_s *ad = (appdata_s *)data;

	if (ad->prev_view == SEARCH_VIEW)
		return;

	init_view(ad);
	ad->prev_view = SEARCH_VIEW;

	char edj_path[PATH_MAX] = {0, };
	app_get_resource(MAIN_VIEW_EDJ_FILE, edj_path, (int)PATH_MAX);

	ad->search_view_layout = elm_layout_add(ad->nf);
	elm_layout_file_set(ad->search_view_layout, edj_path, "search-view");
	evas_object_size_hint_weight_set(ad->search_view_layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(ad->search_view_layout, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_show(ad->search_view_layout);

	ad->search_entry = __create_entry_layout(ad);
	ad->search_genlist = __create_genlist_layout(ad);

	ad->nf_it = elm_naviframe_item_push(ad->nf, NULL, NULL, NULL, ad->search_view_layout, "tabbar/icon/notitle");
	create_search_toolbar(ad);
}
