/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "util.h"
#include "view.h"

/**
 * @brief Get the path of app resource.
 * @param[in] edj_file_in The path of resource
 * @param[in] edj_path_out The path of resource
 * @param[in] edj_path_max The length of the path
 */
void app_get_resource(const char *edj_file_in, char *edj_path_out, int edj_path_max)
{
	char *res_path = app_get_resource_path();
	if (res_path) {
		snprintf(edj_path_out, edj_path_max, "%s%s", res_path, edj_file_in);
		free(res_path);
	}
}

/**
 * @brief Create an search field.
 * @param[in] parent The Evas_Object to add the search field
 */
Evas_Object *create_editfield(Evas_Object *parent)
{
	Evas_Object *editfield, *entry;

	/*
	 * Searchfield Layout
	 * Create and initialize elm_layout.
	 */
	editfield = elm_layout_add(parent);
	elm_layout_theme_set(editfield, "layout", "searchfield", "singleline");
	evas_object_size_hint_align_set(editfield, EVAS_HINT_FILL, 0.0);
	evas_object_size_hint_weight_set(editfield, EVAS_HINT_EXPAND, 0.0);

	/*
	 * Entry
	 * Create and initialize elm_entry.
	 */
	entry = elm_entry_add(editfield);
	elm_entry_single_line_set(entry, EINA_TRUE);
	elm_entry_scrollable_set(entry, EINA_TRUE);
	elm_entry_input_panel_return_key_type_set(entry, ELM_INPUT_PANEL_RETURN_KEY_TYPE_SEARCH);
	evas_object_show(entry);

	elm_object_part_content_set(editfield, "elm.swallow.content", entry);

	return editfield;
}

/**
 * @brief Create an no-content layout.
 * @param[in] parent The Evas_Object to add the no-content layout
 * @param[in] text The text for the no-content layout
 * @param[in] help_text The help text for the no-content layout
 */
Evas_Object *create_nocontent_layout(Evas_Object *parent, const char *text, const char *help_text)
{
	Evas_Object *layout = elm_layout_add(parent);
	elm_layout_theme_set(layout, "layout", "nocontents", "default");
	evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(layout, EVAS_HINT_FILL, EVAS_HINT_FILL);
	if (text)
		elm_object_part_text_set(layout, "elm.text", text);
	if (help_text)
		elm_object_part_text_set(layout, "elm.help.text", help_text);

	elm_layout_signal_emit(layout, "text,disabled", "");
	elm_layout_signal_emit(layout, "align.center", "elm");
	return layout;
}

/**
 * @brief Swap two places.
 * @param[in] places The places
 * @param[in] left The index of place
 * @param[in] right The index of place
 */
static void __swap_place(place_info_s * places[], int left, int right)
{
	place_info_s *tmp_place = places[left];
	places[left] = places[right];
	places[right] = tmp_place;
}

/**
 * @brief split the places for quick sort.
 * @param[in] places The places
 * @param[in] left The index of place
 * @param[in] right The index of place
 */
static int __partition_quick(place_info_s *places[], int left, int right)
{
	double pivot_value = places[left]->distance;
	int low = left + 1;
	int high = right;

	while (low <= high) {
		while (low <= right && pivot_value >= places[low]->distance)
			low++;
		while (high >= left + 1 && pivot_value <= places[high]->distance)
			high--;
		if (low <= high)
			__swap_place(places, low, high);
	}
	__swap_place(places, left, high);
	return high;
}

/**
 * @brief Sort places out.
 * @param[in] places The places
 * @param[in] left The index of place
 * @param[in] right The index of place
 */
void sort_quick_places(place_info_s *places[], int left, int right)
{
	if (left <= right) {
		int pivot = __partition_quick(places, left, right);
		sort_quick_places(places, left, pivot - 1);
		sort_quick_places(places, pivot + 1, right);
	}
}

/**
 * @brief Remove all POIs.
 * @param[in] ad The application data
 */
void remove_all_pois(appdata_s *ad)
{
	maps_view_remove_all_objects(ad->map_view_handle);

	for (int i = 0; i < ad->total_poi; i++) {
		if (ad->poi_places[i]) {
			free(ad->poi_places[i]);
			ad->poi_places[i] = NULL;
		}
	}
	ad->total_poi = 0;

	Evas_Object *hoversel_layout = elm_object_part_content_get(ad->search_result_layout, "hoversel");
	evas_object_del(hoversel_layout);
}

/**
 * @brief Called with the information about user's consent.
 * @param[in] consented The value of User Consent
 * @param[in] maps_provider The maps provider
 * @param[in] user_data The user data
 */
static void user_consent_cb(bool consented, const char *maps_provider, void *user_data)
{
	dlog_print(DLOG_DEBUG, LOG_TAG, "user_consent_cb : Enter[%s]", maps_provider);

	appdata_s *ad = (appdata_s *)user_data;
	int error = 0;
	bool supported = false;

	if (consented == true)
		maps_service_create(maps_provider, &(ad->maps_service_handle));
	else
		ui_app_exit();

	/* Create a default view */
	if (view_create(ad) != EINA_TRUE)
		return;
	dlog_print(DLOG_ERROR, LOG_TAG, "view_create and create_maps_handle : done");

	/* Set the provider Key to access the services from provider */
	maps_service_set_provider_key(ad->maps_service_handle, HERE_PROVIDER_KEY);

	/* Check if Routing is available */
	error = maps_service_provider_is_service_supported(ad->maps_service_handle,
							MAPS_SERVICE_SEARCH_ROUTE, &supported);
	ad->is_search_route_supported = (error == MAPS_ERROR_NONE) ? supported : false;

	error = maps_service_provider_is_service_supported(ad->maps_service_handle,
							MAPS_SERVICE_SEARCH_PLACE, &supported);
	ad->is_search_place_supported = (error == MAPS_ERROR_NONE) ? supported : false;

	error = maps_service_provider_is_service_supported(ad->maps_service_handle,
							MAPS_SERVICE_REVERSE_GEOCODE, &supported);
	ad->is_reverse_geocode_supported = (error == MAPS_ERROR_NONE) ? supported : false;

	/* push an item to the naviframe */
	ad->nf_it = elm_naviframe_item_push(ad->nf, NULL, NULL, NULL, NULL, "tabbar/icon/notitle");

	if (error == MAPS_ERROR_NONE) {
		/* Create the toolbar */
		if (view_create_toolbar(ad) == NULL)
			dlog_print(DLOG_ERROR, LOG_TAG, "failed to create a toolbar");
	}
}

/**
 * @brief Function will be operated when any provider is pressed.
 * @param[in] data The data to be passed to the callback function
 */
void request_user_consent(void *data)
{
	dlog_print(DLOG_DEBUG, LOG_TAG, "request_user_consent : Enter");

	int error = maps_service_request_user_consent("HERE", user_consent_cb, data);
	if (error != MAPS_ERROR_NONE)
		ui_app_exit();
}

/**
 * @brief Create handles of maps view and maps service.
 * @param[in] ad The application data
 */
void destroy_maps_handle(appdata_s *ad)
{
	if (ad->map_view_handle) {
		int error = maps_view_destroy(ad->map_view_handle);
		if (error != MAPS_ERROR_NONE)
			dlog_print(DLOG_ERROR, LOG_TAG, "maps_view_destroy: fail [%d]", error);
		else
			dlog_print(DLOG_ERROR, LOG_TAG, "maps_view_destroy: success");
		ad->map_view_handle = NULL;
	}
	if (ad->maps_service_handle) {
		int error = maps_service_destroy(ad->maps_service_handle);
		if (error != MAPS_ERROR_NONE)
			dlog_print(DLOG_ERROR, LOG_TAG, "maps_service_destroy: fail [%d]", error);
		else
			dlog_print(DLOG_ERROR, LOG_TAG, "maps_service_destroy: success");
		ad->maps_service_handle = NULL;
	}
}

/**
 * @brief Add the markers.
 * @param[in] view The handle of maps view
 * @param[in] place The place
 * @param[in] selected The value about checking the selected place
 */
void add_marker(maps_view_h view, place_info_s *place, bool selected)
{
	dlog_print(DLOG_ERROR, LOG_TAG, "add_marker: [%f,%f]", place->location.latitude, place->location.longitude);

	char image_path[PATH_MAX] = {0, };
	if (selected == true)
		app_get_resource(CUR_POI_MARKER_IMAGE, image_path, (int)PATH_MAX);
	else
		app_get_resource(POI_MARKER_IMAGE, image_path, (int)PATH_MAX);

	maps_coordinates_h coord;
	maps_coordinates_create(place->location.latitude, place->location.longitude, &coord);
	maps_view_object_create_marker(coord, image_path, MAPS_VIEW_MARKER_PIN, &(place->marker));
	maps_view_add_object(view, place->marker);

	if (selected == true) {
		maps_view_object_marker_set_z_order(place->marker, 100);
		maps_view_object_marker_resize(place->marker, 80, 80);
	}
}

/**
 * @brief Update the markers.
 * @param[in] ad The application data
 * @param[in] current_poi The index of selected poi
 */
void update_poi_markers(appdata_s *ad, int current_poi)
{
	maps_view_remove_all_objects(ad->map_view_handle);
	maps_view_set_zoom_level(ad->map_view_handle, 15);

	for (int i = 0; i < ad->total_poi && i < MAX_PLACES; i++) {
		if (i == current_poi) {
			add_marker(ad->map_view_handle, ad->poi_places[i], true);

			maps_coordinates_h maps_coord = NULL;
			maps_coordinates_create(ad->poi_places[i]->location.latitude, ad->poi_places[i]->location.longitude, &maps_coord);
			maps_view_set_center(ad->map_view_handle, maps_coord);
			maps_coordinates_destroy(maps_coord);
		} else
			add_marker(ad->map_view_handle, ad->poi_places[i], false);
	}
}
