/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <tizen.h>
#include <dlog.h>
#include <app.h>
#include <efl_extension.h>
#include <Elementary.h>

#include "$(appName).h"
#include "view.h"
#include "main_view.h"
#include "search_view.h"
#include "route_view.h"
#include "util.h"

static struct view_info {
	Evas_Object *win;
	Evas_Object *conform;
} s_info = {
	.win = NULL,
	.conform = NULL,
};

static void _win_delete_request_cb(void *data, Evas_Object *obj, void *event_info);
static Evas_Object *view_create_naviframe(Evas_Object *conform);

/**
 * @brief Creates essential objects: window, conformant.
 */
Eina_Bool view_create(void *user_data)
{
	appdata_s *ad = (appdata_s *)user_data;

	elm_config_accel_preference_set("opengl");

	/* Create the window */
	s_info.win = view_create_win(PACKAGE);
	if (s_info.win == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to create a window.");
		return EINA_FALSE;
	}

	ad->win = s_info.win;

	/* Create the conformant */
	s_info.conform = view_create_conformant_without_indicator(s_info.win);
	if (s_info.conform == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to create a conformant");
		return EINA_FALSE;
	}

	/* Create the naviframe */
	ad->nf = view_create_naviframe(s_info.conform);
	if (ad->nf == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to create a naviframe");
		return EINA_FALSE;
	}

	/* Show the window after main view is set up */
	evas_object_show(s_info.win);
	return EINA_TRUE;
}

/**
 * @brief Creates a basic window named package.
 * @param[in] pkg_name Name of the window
 */
Evas_Object *view_create_win(const char *pkg_name)
{
	Evas_Object *win = NULL;

	/*
	 * Window
	 * Create and initialize elm_win.
	 * elm_win is mandatory to manipulate the window.
	 */
	win = elm_win_util_standard_add(pkg_name, pkg_name);
	elm_win_conformant_set(win, EINA_TRUE);
	elm_win_autodel_set(win, EINA_TRUE);

	/* Rotation setting */
	if (elm_win_wm_rotation_supported_get(win)) {
		int rots[4] = { 0, 90, 180, 270 };
		elm_win_wm_rotation_available_rotations_set(win, (const int *)(&rots), 4);
	}

	evas_object_smart_callback_add(win, "delete,request", _win_delete_request_cb, NULL);

	return win;
}

/**
 * @brief Creates a conformant without indicator for wearable app.
 * @param[in] win The object to which you want to set this conformant
 * Conformant is mandatory for base GUI to have proper size
 */
Evas_Object *view_create_conformant_without_indicator(Evas_Object *win)
{
	/*
	 * Conformant
	 * Create and initialize elm_conformant.
	 * elm_conformant is mandatory for base GUI to have proper size
	 * when indicator or virtual keypad is visible.
	 */
	Evas_Object *conform = NULL;

	if (win == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "window is NULL.");
		return NULL;
	}

	conform = elm_conformant_add(win);
	evas_object_size_hint_weight_set(conform, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_win_resize_object_add(win, conform);

	evas_object_show(conform);

	return conform;
}

/**
 * @brief Destroys window and frees its resources.
 */
void view_destroy(void)
{
	if (s_info.win == NULL)
		return;

	evas_object_del(s_info.win);
}

/**
 * @brief Function will be operated when registered event is triggered.
 * @param[in] data The data to be passed to the callback function
 * @param[in] obj The Evas object handle to be passed to the callback function
 * @param[in] event_info The system event information
 */
static void _win_delete_request_cb(void *data, Evas_Object *obj, void *event_info)
{
	ui_app_exit();
}

/**
 * @brief Creates a naviframe
 * @param[in] conform The object to which you want to set this naviframe
 */
static Evas_Object *view_create_naviframe(Evas_Object *conform)
{
	/*
	 * Naviframe
	 * Create and initialize elm_naviframe.
	 */
	Evas_Object *nf = NULL;

	if (conform == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "conform is NULL.");
		return NULL;
	}

	nf = elm_naviframe_add(conform);
	elm_object_part_content_set(conform, "elm.swallow.content", nf);
	eext_object_event_callback_add(nf, EEXT_CALLBACK_BACK, _win_delete_request_cb, NULL);
	eext_object_event_callback_add(nf, EEXT_CALLBACK_MORE, eext_naviframe_more_cb, NULL);

	evas_object_show(nf);

	return nf;
}

/**
 * @brief Creates a toolbar
 * @param[in] ad The application data
 */
Evas_Object *view_create_toolbar(appdata_s *ad)
{
	/*
	 * Toolbar
	 * Create and initialize elm_toolbar.
	 */
	ad->toolbar = elm_toolbar_add(ad->nf);
	elm_object_style_set(ad->toolbar, "tabbar");
	elm_toolbar_shrink_mode_set(ad->toolbar, ELM_TOOLBAR_SHRINK_EXPAND);
	elm_toolbar_transverse_expanded_set(ad->toolbar, EINA_TRUE);
	elm_toolbar_select_mode_set(ad->toolbar, ELM_OBJECT_SELECT_MODE_DEFAULT);

	elm_toolbar_item_selected_set(elm_toolbar_item_append(ad->toolbar, NULL, "Map", create_main_view, ad), EINA_TRUE);
	elm_toolbar_item_append(ad->toolbar, NULL, "POI", create_search_view, ad);
	elm_toolbar_item_append(ad->toolbar, NULL, "Route", create_route_view, ad);

	elm_object_item_part_content_set(ad->nf_it, "tabbar", ad->toolbar);

	return ad->toolbar;
}
